// GSMSMS Library
// Copyright (c) 2014, 2015, 2016, 2017, 2018, 2019 by Haute Solutions / Ed Van Every (www.haute-solutions.com)
//
// FUNCTION: This Library is intended to provide GSM SMS and GPS functionality
//
// HARDWARE: This LIbrary Supports the following GSM chipsets
//						
//					 SIMCOM SIM808
//					 SIMCOM SIM900
//					 SIMCOM SIM5320
//					 SIMCOM SIM7100
//					 SIMCOM SIM7600
//
// As Implemented on the following Boards:
//
//           Itead GBoard (SIM900 on Pins 2/33, Power on Pin 6)
//           Haute Solutions GBoardHS (SIM900 on UART2, Power on Pin 6)
//           Haute Solutions EGGBoard (SIM900 on UART2, Power on Pin 6)
//           Haute Solutions EGGBoard 3G (SIM5320 on UART1, Power on Pin 6)
//           Haute Solutions MeGGaBoard (SIM808 on UART1, Power on Pin 46)
//           Haute Solutions MeGGaBoard 3G (SIM5320 on UART1, Power on Pin 46)
//           Haute Solutions ESP32G (SIM7600 on UART1, Power on Pin 4)
//           Haute Solutions ESP32GE (SIM7600 on UART1, Power on Pin 18)
//
// SOFTWARE: None Additional
//
// LICENSE:  This library is free software: you can redistribute it and/or modify
//           it under the terms of the GNU General Public License as published by
//           the Free Software Foundation, either version 3 of the License, or
//           (at your option) any later version.
//
//           This program is distributed in the hope that it will be useful,
//           but WITHOUT ANY WARRANTY; without even the implied warranty of
//           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//           GNU General Public License for more details.
//           <http://www.gnu.org/licenses/>.
//

#define GSMSMS_LIBVER "4.3"

#ifndef GSMSMS_h
#define GSMSMS_h

//  EITHER GSM_SW_SERIAL, GSM_HW_SERIAL, or GSM_ESP32_SERIAL must be defined!!!
//  - GBoard uses GSM_SW_SERIAL (SIM900 on Pins 2/33,  Power on Pin 6)
//  - GBoardHS uses GSM_HW_SERIAL (SIM900 on UART2, Power on Pin 6)
//  - EGGBoard uses GSM_HW_SERIAL (SIM900 on UART2, Power on Pin 6)
//  - EGGBoard 3G uses GSM_HW_SERIAL (SIM5320 on UART1, Power on Pin 6)
//  - MeGGaBoard uses GSM_HW_SERIAL (SIM808 on UART1, Power on Pin 46)
//  - MeGGaBoard 3G uses GSM_HW_SERIAL (SIM5320 on UART1, Power on Pin 46)
//  - ESP32G uses GSM_HW_SERIAL (SIM7600 on UART1, Power on Pin 4)
//  - ESP32GE uses GSM_HW_SERIAL (SIM7600 on UART1, Power on Pin 18)
#define GBOARD				9000
#define GBOARDHS			9001
#define EGGBOARD			9002
#define EGGBOARD3G		9003
#define MEGGABOARD		9004 
#define MEGGABOARD3G	9005
#define ESP324G				9006
#define ESP324GE			9007

// We'll try and do this automatically...

#if defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168__)
#define GSM_SW_SERIAL
#endif 

#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
#define GSM_HW_SERIAL
#endif 

#if defined(ESP32)
#define GSM_ESP32_SERIAL
#endif 

// Otherwise comment out the above and uncomment ONE of the following...
//#define GSM_HW_SERIAL
//#define GSM_SW_SERIAL
//#define GSM_ESP32_SERIAL

// GSM GPS functions are included by default
// To disable GSM GPS functions, just comment out the line below
#define GSM_GPS

// ------------------------------------------------------

#include "Arduino.h"

#define GSMBUFFSIZE 512		// Originally 200!  Note that TWO Buffers are created using this value...

class GSMSMS
{
	public:
		GSMSMS(int OnPin, int UART=2, int OnPulse=4000, int OnWait=13000);		// GBoardHS and EGGBoard UART default
		int  SendCmd(const char *AT_cmd_string);
		int  SendCmdParams(const char *AT_cmd_string, int start_comm_tmout, int max_interchar_tmout, const char *response_string, int no_of_attempts);
		int  WaitResp(unsigned long start_comm_tmout, unsigned long max_interchar_tmout, const char *response_string);
		int  Init(long baud_rate=115200, int debug=0);
		int  Init1(long baud_rate=115200, int debug=0);
		int  Init2(long baud_rate=115200, int debug=0);
		int  Init3(long baud_rate=115200);
		void TurnOn(int ms_high, int ms_low);
		void TurnOff();
		int  CheckModem();
		long DetectBaud();
		int  SetBaud(long baud_rate);
		void SetParams();
		int  GetSMS(char *number, char *msg, int nsize, int msize); 
		int  SendSMS(const char *number, const char *msg);
		int  PrepToSendSMS(const char *number);
		int  FlushSMS();
		void DeleteAllSMS();
		int  GetSignalQuality();
		int  GetTime(char *time);
		int  SIMInserted();
		int  RegisteredOnNetwork();
		int  RegisteredOnNetworkExt(int mode);
		int  GetGService();
		int  GetSubscriberNumber(char *snum);
		int  GetSimCom();
		int  GetChipset(char *chipinfo=NULL);
		int  GetIMEI(char *imei);
		int  GetICCID(char *iccid);
		int  GetIMSI(char *imei);
		int  GetNetworkOperator(char *Operator);
		int  GetNetworkMode(char *Mode);
		int  GetServiceCenter(char *SCA);
		int  GetPreferredMode();
		int  SetPreferredMode(int mode);
		int  GetATResult(const char *acmd, const char *akey, char *avalue, int timeout);
		int  TrimMessage(char *Msg, char StartDelim, char EndDelim);
#ifdef GSM_GPS
		int  EnableGPS();
		int  DisableGPS();
		int  GetGPSPosition(char *latitude, char *longitude);
		int  GetGPSPosition5320(char *latitude, char *longitude);
		int  GetGPSPosition808(char *latitude, char *longitude);
#endif
		void Begin(long Baud_Rate);
		void SerialReset();
		int  Available();
		int  Read();
		int  Write(int c);
		long Print(const char *text);
		long Print(int i);
		long Println(const char *text);
		long Println(int i);
		char GSMCommBuff[GSMBUFFSIZE];   // GSM Communications Buffer (Public)
	private:
		int _OnPin;
		int _UART;
		int _Board;
		int _OnPulse;
		int _OnWait;
		int _Debug;
		int _LastBaud;
		int _SimCom;                     // Chipset type (0=Unknown, 900=SIM900, 808=SIM808, 5320=SIM5320A, 7100=SIM7100A, 7600=SIM7600A)
		int _RXPin;
		int _TXPin;
		int _GService;
		const char *_NetMode[25] = { 
			"No Service",
			"GSM",
			"GPRS",
			"EGPRS (EDGE)",
			"WCDMA",
			"HSDPA only(WCDMA)",
			"HSUPA only(WCDMA)",
			"HSPA (HSDPA and HSUPA, WCDMA)",
			"LTE",
			"TDMS - CDMA",
			"TDS - HSDPA only",
			"TDS - HSUPA only",
			"TDS - HSPA (HSDPA and HSUPA)",
			"CDMA",
			"EVDO",
			"HYBRID (CDMA and EVDO)",
			"1XLTE(CDMA and LTE)",
			"Undefined",
			"Undefined",
			"Undefined",
			"Undefined",
			"Undefined",
			"Undefined",
			"eHRPD",
			"HYBRID(CDMA and eHRPD)"};
};

#endif





